"""
    Shortcuts for models.
"""
__all__ = ['DatabaseModel', 'Dat', 'Seed', 'System']

from .datfile import DatabaseModel
from .datfile import Dat
from .datfile import Seed
from .datfile import System
